/*
 * Copyright 2012 - Doxia :: Include Macro - Juergen Kellerer
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package org.tinyjee.maven.dim;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Holds macro presets as definable within {@link InitializeMacroMojo}.
 *
 * @author Juergen_Kellerer, 2012-02-21
 */
public class MacroPreset {

	private static final String KEY_PREFIX = "org.tinyjee.maven.dim.preset.";

	private String name = "";
	private Map<String, String> properties = new HashMap<String, String>();

	public MacroPreset() {
	}

	public MacroPreset(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public Map<String, String> getProperties() {
		return properties;
	}

	void encodeTo(Properties props) {
		props.setProperty(KEY_PREFIX + name, "true");
		for (Map.Entry<String, String> entry : properties.entrySet())
			props.put(KEY_PREFIX + name + '.' + entry.getKey(), entry.getValue());
	}

	static MacroPreset decodeFrom(String name, Properties props) {
		String prefix = KEY_PREFIX + name;
		if (!Boolean.getBoolean(prefix)) return null;

		final MacroPreset preset = new MacroPreset(name);
		prefix += ".";
		for (Map.Entry<Object, Object> entry : props.entrySet()) {
			final String key = entry.getKey().toString();
			if (key.startsWith(prefix))
				preset.getProperties().put(key.substring(prefix.length()), String.valueOf(entry.getValue()));
		}

		return preset;
	}
}
