/*
 * Copyright 2012 - Doxia :: Include Macro - Juergen Kellerer
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package org.tinyjee.maven.dim;

import org.apache.maven.doxia.logging.Log;
import org.tinyjee.maven.dim.spi.Globals;
import org.tinyjee.maven.dim.spi.RequestParameterTransformer;

import java.util.Map;

/**
 * Implements a parameter transformer that can be configured with MacroPresets coming from {@link InitializeMacroMojo}.
 *
 * @author Juergen_Kellerer, 2012-02-21
 */
public class MacroPresetParameterTransformer implements RequestParameterTransformer {

	static final String KEY_PRESET = "preset";
	static final String DEFAULT_PRESET = "default";

	public void transformParameters(Map<String, Object> requestParams) {
		Object presetName = requestParams.get(KEY_PRESET);

		if (presetName == null) {
			presetName = DEFAULT_PRESET;
			if (MacroPreset.decodeFrom(presetName.toString(), System.getProperties()) == null)
				return;
		}

		final String name = presetName.toString();
		final Log logger = Globals.getLog();
		final boolean debug = logger.isDebugEnabled();
		if (debug) {
			logger.debug("Looking for after named properties preset '" + name + "', " +
					"properties before applying preset are: " + requestParams);
		}

		final MacroPreset preset = MacroPreset.decodeFrom(name, System.getProperties());
		if (preset != null) {
			final Map<String, String> properties = preset.getProperties();
			for (Map.Entry<String, String> entry : properties.entrySet()) {
				if (!requestParams.containsKey(entry.getKey()))
					requestParams.put(entry.getKey(), entry.getValue());
			}
		} else {
			logger.error("Did not find a defined properties preset that is named: '" + name + '\'');
		}

		if (debug) logger.debug("Properties after applying preset are: " + requestParams);
	}
}
