/*
 * Copyright 2012 - Doxia :: Include Macro - Juergen Kellerer
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package org.tinyjee.maven.dim.extensions;

import org.apache.maven.doxia.logging.Log;
import org.tinyjee.maven.dim.IncludeMacroSignature;
import org.tinyjee.maven.dim.spi.Globals;

import javax.script.*;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Map;

/**
 * Implements the JSR-223 bindings.
 *
 * @author Juergen_Kellerer, 2012-02-27
 */
public class ScriptExecutorJSR223 implements ScriptInvoker.ScriptExecutor {

	private static final long serialVersionUID = 6582675587269002645L;

	final ScriptEngineManager manager = new ScriptEngineManager();

	public void execute(Map<String, Object> context, String scriptExtension, String scriptContent, String scriptName) throws Exception {
		final Bindings bindings = new SimpleBindings(context);
		execute(bindings, scriptExtension, scriptContent, scriptName);
	}

	protected void execute(Bindings bindings, String scriptExtension, String scriptContent, String scriptName) throws Exception {
		final Log log = Globals.getLog();
		final StringWriter errorWriter = new StringWriter(), outputWriter = new StringWriter();

		final ScriptEngine engine = createEngine(bindings, scriptExtension, scriptName, outputWriter, errorWriter);
		try {
			engine.eval(scriptContent);
		} catch (Exception e) {
			log.error("Failed executing script '" + scriptName + "':\n----\n" + scriptContent + "\n----\n, caused by: ", e);
			throw e;
		} finally {
			Object sourceContent = bindings.get(IncludeMacroSignature.PARAM_SOURCE_CONTENT);
			if (outputWriter.getBuffer().length() > 0 && sourceContent == null) {
				bindings.put(IncludeMacroSignature.PARAM_SOURCE_CONTENT, sourceContent = outputWriter.toString());
				if (bindings.get(IncludeMacroSignature.PARAM_VERBATIM) == null) bindings.put(IncludeMacroSignature.PARAM_VERBATIM, false);
			}

			if (errorWriter.getBuffer().length() > 0) {
				String content = errorWriter.toString();
				log.error(content);

				content = "ERROR:" + content.replace("\n", "\nERROR:");
				if (sourceContent != null) content = content + "\n\n" + sourceContent;
				bindings.put(IncludeMacroSignature.PARAM_SOURCE_CONTENT, content);
			}
		}
	}

	protected ScriptEngine createEngine(Bindings bindings, String scriptExtension, String scriptName,
	                                    StringWriter outputWriter, StringWriter errorWriter) {
		ScriptEngine engine = manager.getEngineByExtension(scriptExtension);
		if (engine == null) engine = manager.getEngineByName(scriptExtension);
		if (engine == null) engine = manager.getEngineByMimeType(scriptExtension);

		if (engine == null) {
			throw new IllegalArgumentException("No script engine exists that is named '" + scriptExtension + "' " +
					"or processes scripts if this file extension.");
		}

		engine.setBindings(bindings, ScriptContext.ENGINE_SCOPE);
		engine.put(ScriptEngine.FILENAME, scriptName);

		engine.getContext().setWriter(new PrintWriter(outputWriter));
		engine.getContext().setErrorWriter(new PrintWriter(errorWriter));

		return engine;
	}
}
