package org.tinyjee.maven.dim.sources;

import org.apache.velocity.tools.config.DefaultKey;
import org.tinyjee.maven.dim.spi.Globals;
import org.tinyjee.maven.dim.spi.ResourceResolver;
import org.tinyjee.maven.dim.utils.PrintableMap;

import java.util.Map;

import static org.tinyjee.maven.dim.IncludeMacroSignature.PARAM_SOURCE_CLASS;

/**
 * Implements a custom velocity tool that can be used to invoke other source classes.
 * <p/>
 * Usage within a template:<code><pre>
 * #set ($result = $invoker.clear().put('paramA', 'value').invokeClass('my.package.MyClass'))
 * Result: $result.get("key")
 * </pre></code>
 *
 * @author Juergen_Kellerer, 2011-10-14
 */
@DefaultKey("invoker")
public class VelocitySourceClassInvocationTool extends AbstractParametricVelocityTool<VelocitySourceClassInvocationTool> {
	/**
	 * Invokes the given 'source-class' compatible helper class.
	 *
	 * @param className the name of the class to invoke.
	 * @return The parameters that were generated by the class.
	 */
	public Map<String, Object> invokeClass(String className) {
		Map<String, String> parameters = nextInvocationParameters();
		parameters.put(PARAM_SOURCE_CLASS, className);

		SourceClassAdapter classAdapter = new SourceClassAdapter(Globals.getInstance().getBasedir(), parameters, className);
		return new PrintableMap<String, Object>(classAdapter.getContent().getParameters(), "Result-Of[" + className + ']');
	}

	/**
	 * Resolves the given 'source-class' providing access to use reflection.
	 *
	 * @param className the name of the class to resolve.
	 * @return The Class instance the resolved.
	 */
	public Class<?> resolveClass(String className) {
		return ResourceResolver.resolveClass(Globals.getInstance().getBasedir(), className);
	}
}
