package org.tinyjee.maven.dim.spi;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

/**
 * Implements a Source that returns fixed content.
 *
 * @author Juergen_Kellerer, 2011-10-23
 */
public class FixedContentSource extends AbstractSource {

	/**
	 * Defines an unknown url that may be used as substitution for 'null' in services that are not null-pointer safe.
	 */
	public static final URL UNKNOWN;

	static {
		try {
			UNKNOWN = new URL("file:///unknown");
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}
	}

	private final String content;

	/**
	 * Constructs a new instance of FixedContentSource with the given content.
	 *
	 * @param content    The fixed content to wrap into a source implementation.
	 * @param parameters the macro parameters assigned with this source.
	 */
	public FixedContentSource(String content, Map<String, Object> parameters) {
		super(null, parameters);
		this.content = content;
	}

	@Override
	public Content getContent() {
		return new SourceContent() {
			@Override
			public Reader openReader() throws IOException {
				return new StringReader(content);
			}
		};
	}
}
